"""
The start command can be used to start any or all protocol servers.  To start
all of the server, execute the start command with no paramters.  To bring up
individual servers, list them on the command line.
"""

import os, sys

from Ft.Server.Server import FtServerServerException, Error
from Ft.Server.Core import Constants
from Ft.Server.Core.Lib import ConfigFile

def Run(serverNames=None, configFile=None, coreId=None,display=1):
    from Ft.Server.Protocols import StartMode, Launcher

    properties = ConfigFile.ReadProperties(fileName=configFile)
    coreId = coreId or Constants.DEFAULT_CORE_ID

    serverDefs = {}
    for defs in properties.values():
        serverDefs.update(defs)

    if not serverNames:
        for name, prop in serverDefs.items():
            if prop.get('StartMode') == StartMode.AUTOMATIC:
                Launcher.Launcher(name, prop, display,configFileName=configFile,coreId=coreId).start()
    else:
        for name in serverNames:
            prop = serverDefs.get(name)
            if not prop:
                raise FtServerServerException(Error.UNKNOWN_SERVER, server=name)
            Launcher.Launcher(name, prop, display,configFileName=configFile,coreId=coreId).start()
    return


def TextStart(options, args):
    try:
        serverNames = args.get('serverName')
        configFile = options.get('config-file')
        coreId = options.get('core-id', 'Core')
        Run(serverNames, configFile, coreId)
    except FtServerServerException, e:
        print e
    return




def Register():
    from Ft.Lib.CommandLine import Options, Command, Arguments

    cmd = Command.Command('start',
                          "Start protocol server(s)",
                          'PythonHttpServer',
                          __doc__,
                          function=TextStart,
                          arguments = [Arguments.ZeroOrMoreArgument('serverName',
                                                                    'Name of server to wuery',
                                                                    str),
                                       ],
                          authenticate='start',
                          )
    return cmd

