
from Ft.Server.Server import FtServerServerException, Error
from Ft.Server.Core import Constants
import FtServerResources__POA, FtServerResources
import FtssTypes
from Ft.Server.Protocols.CORBA.FtOmniorb.Lib import CorbaUtil, ExceptionUtil
from Ft.Server.Protocols.CORBA.FtOmniorb import OoAclObjectImp

from Ft.Rdf import Statement

class ResourceImp(FtServerResources__POA.ResourceIf,OoAclObjectImp.AclObjectImp):



    def getUri(self):
        return self._runSafe(self.resource.getUri,())


    def getCreationDate(self):
        return self._runSafe(self.resource.getCreationDate,(),int)

    def getLastModifiedDate(self):
        return self._runSafe(self.resource.getLastModifiedDate,(),int)


    def getResourceType(self):
        return self._runSafe(self.resource.getResourceType,(),g_resourceTypeMap.get)

    def isResourceType(self,testType):
        return self._runSafe(self.resource.isResourceType,(g_inverseResourceTypeMap[testType],))



    def hasRawFile(self,uri):
        return self._runSafe(self.resource.hasRawFile,(uri,))

    def hasDocument(self,uri):
        return self._runSafe(self.resource.hasDocument,(uri,))

    def hasContainer(self,uri):
        return self._runSafe(self.resource.hasContainer,(uri,))

    def hasAlias(self,uri):
        return self._runSafe(self.resource.hasAlias,(uri,))

    def hasResource(self,uri):
        return self._runSafe(self.resource.hasResource,(uri,))


    def createContainer(self,uri,createParents):
        return self._runSafe(self.resource.createContainer,(uri,createParents),self._mapResource)

    def createRawFile(self,uri,imt,src):
        return self._runSafe(self.resource.createRawFile,(uri,imt,src),self._mapResource)

    def createDocument(self,docDefName,uri,src):
        return self._runSafe(self.resource.createDocument,(docDefName,uri,src),self._mapResource)

    def fetchContainer(self,uri):
        return self._runSafe(self.resource.fetchContainer,(uri,),self._mapResource)

    def fetchRawFile(self,uri):
        return self._runSafe(self.resource.fetchRawFile,(uri,),self._mapResource)

    def fetchAlias(self,uri):
        return self._runSafe(self.resource.fetchAlias,(uri,),self._mapResource)

    def fetchDocument(self,uri):
        return self._runSafe(self.resource.fetchDocument,(uri,),self._mapResource)

    def fetchResource(self,uri,traverseAliases):
        return self._runSafe(self.resource.fetchResource, (uri,traverseAliases),self._mapResource)

    def fetchResourceInformation(self,uri):
        return self._runSafe(self.resource.fetchResourceInformation, (uri,),self._mapResourceInformation)

    def deleteContainer(self,uri):
        return self._runSafe(self.resource.deleteContainer,(uri,))

    def deleteDocument(self,uri):
        return self._runSafe(self.resource.deleteDocument,(uri,))

    def deleteAlias(self,uri):
        return self._runSafe(self.resource.deleteAlias,(uri,))

    def getModel(self):
        return self._runSafe(self.resource.getModel,(),self._mapModel)

    def getAllDocumentDefinitionUris(self):
        return self._runSafe(self.resource.getAllDocumentDefinitionUris,())

    def addAlias(self,uri):
        return self._runSafe(self.resource.addAlias,(uri,),self._mapResource)

    def removeAlias(self,uri):
        return self._runSafe(self.resource.removeAlias,(uri,))


    def delete(self):
        ###NOTE: Should we invalidate the object in some way?
        return self._runSafe(self.resource.delete,())


    #Private Helpers
    def _mapResourceInformation(self,internal):
        for name,(value,func) in g_defaultResourceInformation.items():
            if not hasattr(internal,name):
                setattr(internal,name,value)
            else:
                setattr(internal,name,func(getattr(internal,name)))
                
        setattr(internal,'resourceType',g_resourceTypeMap[internal.resourceType])
        #for name,value in internal.__dict__.items():
        #    print "%s %s %s" % (name,value,type(value))
        return internal

    def _mapResource(self,internal):
        if internal.getResourceType() == Constants.ResourceType.CONTAINER:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoContainerImp
            res = OoContainerImp.ContainerImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.USER_SYSTEM_CONTAINER:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoSystemUserContainerImp
            res = OoSystemUserContainerImp.SystemUserContainerImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.SUPER_USER_SYSTEM_CONTAINER:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoSystemSuperUserContainerImp
            res = OoSystemSuperUserContainerImp.SystemSuperUserContainerImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.GROUP_SYSTEM_CONTAINER:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoSystemGroupContainerImp
            res = OoSystemGroupContainerImp.SystemGroupContainerImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.DOCDEF_SYSTEM_CONTAINER:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoSystemDocDefContainerImp
            res = OoSystemDocDefContainerImp.SystemDocDefContainerImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.FILE:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoRawFileImp
            res = OoRawFileImp.RawFileImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.XML_DOCUMENT:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoXmlDocumentImp
            res = OoXmlDocumentImp.XmlDocumentImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.REPOSITORY:
            return self._repo._this()
        elif internal.getResourceType() == Constants.ResourceType.DOCUMENT_DEFINITION:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoDocumentDefinitionImp
            res = OoDocumentDefinitionImp.DocumentDefinitionImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.XSLT_DOCUMENT:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoXmlDocumentImp
            res = OoXmlDocumentImp.XmlDocumentImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.SCHEMATRON_DOCUMENT:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoXmlDocumentImp
            res = OoXmlDocumentImp.XmlDocumentImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.ALIAS:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoAliasImp
            res = OoAliasImp.AliasImp(internal,self._repo)
        elif internal.getResourceType() == Constants.ResourceType.USER:
            from Ft.Server.Protocols.CORBA.FtOmniorb import OoUserImp
            res = OoUserImp.UserImp(internal,self._repo)
        else:
            raise "Finish %s" % internal
        return CorbaUtil.ActivateObject(res)

    def _mapResources(self,reses):
        new = []
        for r in reses:
            new.append(self._mapResource(r))
        return new

    def _mapModel(self,model):
        from Ft.Server.Protocols.CORBA.FtOmniorb import OoRdfModelImp
        model = OoRdfModelImp.RdfModelImp(model)
        return CorbaUtil.ActivateObject(model)

    def _mapRdfStatements(self,stmts):
        newStmts = []
        for s in stmts:
            newStmts.append(FtssTypes.RdfStatement(s.subject,
                                                  s.predicate,
                                                  s.object,
                                                  s.uri,
                                                  s.domain))
        return newStmts
            


g_resourceTypeMap = {}
g_inverseResourceTypeMap = {}

for e in FtssTypes.ResourceTypeEnum._items:
    g_resourceTypeMap[e._v] = e
    g_inverseResourceTypeMap[e] = e._v



g_defaultResourceInformation = {'uri':("",str),
                                'creationDate':(0,int),
                                'lastModifiedDate':(0,int),
                                'canRead':(0,int),
                                'canWrite':(0,int),
                                'numberChildren':(0,int),
                                'seperator':('',str),
                                'referenceUri':('',str),
                                'size':(0,int),
                                'imt':('',str),
                                'creationFlags':(0,int),
                                'members':([],list),
                                'password':('',str)


                                }
