#include "domlette.h"

/*External Interfaces*/

PyObject *g_documentTypeNodeType; // added by Brendan Larson 9/12/2002 for Mac OS X 10.1.x compiler compatiblity


static struct PyMethodDef DocType_methods[] = { NODE_METHODS,
  {NULL,     NULL}      /* sentinel */
};


/*
  Type Interfaces
*/

static PyObject *doctype_getattr(PyDocTypeObject *self, char *name)
{
  PyObject *rt = NULL;

  if (!strcmp(name, "name")) {
    rt = (PyObject *)self->nodeName;
  }
  else if (!strcmp(name,"publicId")) {
    rt = (PyObject *)self->publicId;
  }
  else if (!strcmp(name,"systemId")) {
    rt = (PyObject *)self->systemId;
  }
  else if (!strcmp(name,"nodeName")) {
    rt = (PyObject *)self->nodeName;
  }
  else if (!strcmp(name,"internalSubset")) {
    rt = (PyObject *)self->internalSubset;
  }
  else if (!strcmp(name,"nodeType")) {
    rt = g_documentTypeNodeType;
  }
  else if (!strcmp(name,"entities")) {
    return PyDict_New();
  }
  else if (!strcmp(name,"notations")) {
    return PyDict_New();
  }
  if (rt) {
    Py_INCREF(rt);
    return rt;
  }

  return node_getattr((PyNodeObject*)self,name, DocType_methods);
}

static int doctype_setattr(PyDocTypeObject *self, char *name, PyObject *v)
{
  /* Set attribute 'name' to value 'v'. v==NULL means delete */
  if (v == NULL) {
    PyErr_Format(PyExc_AttributeError, 
                 "Cannot delete attribute '%.400s' on '%.50s' object",
                 name, self->ob_type->tp_name);
  } else {
    PyErr_Format(PyExc_AttributeError,
                 "Cannot set attribute '%.400s' on '%.50s' object", 
                 name, self->ob_type->tp_name);
  }
  return -1;
}

static void
doctype_dealloc(PyDocTypeObject *node)
{
  Py_XDECREF(node->publicId);
  node->publicId = NULL;
  Py_XDECREF(node->systemId);
  node->systemId = NULL;
  Py_XDECREF(node->internalSubset);
  node->internalSubset = NULL;
  Py_XDECREF(node->nodeName);
  node->nodeName = NULL;
  node_dealloc((PyNodeObject*)node);
}


static PyObject *doctype_repr(PyDocTypeObject *doctype)
{
  char buf[256];
  sprintf(buf, "<cDocumentType at %p>", doctype);
  return PyString_FromString(buf);
}

static int doctype_print(PyDocTypeObject *v, FILE *fp, int flags)
{
  PyObject *repr = doctype_repr(v);
  fprintf(fp, PyString_AS_STRING(repr));
  Py_DECREF(repr);
  return 0;
}

static int doctype_clear(PyDocTypeObject *self)
{
  /*Called when we need to break cycles*/
  return node_clear((PyNodeObject *)self);
}

static int doctype_traverse(PyDocTypeObject *self, visitproc visit, void *arg)
{
  return node_traverse((PyNodeObject *)self,visit,arg);
}


PyTypeObject PyDomletteDocType_Type = {
    PyObject_HEAD_INIT(0)
    0,
    "cDomlette.DocType",
    GC_TP_BASICSIZE(PyDocTypeObject),
    0,
    (destructor)doctype_dealloc,   /*tp_dealloc*/
    (printfunc)doctype_print,  /*tp_print*/
    (getattrfunc)doctype_getattr,  /*tp_getattr*/
    (setattrfunc)doctype_setattr,  /*tp_setattr*/
    0,                          /*tp_compare*/
    (reprfunc)doctype_repr,     /*tp_repr*/
    0,                          /*tp_as_number*/
    0,              /*tp_as_sequence*/
    0,              /*tp_as_mapping*/
    0,                             /*tp_hash*/
    0,          /*tp_call*/
    0,          /*tp_str*/
    0,                      /*tp_getattro*/
    0,          /*tp_setattro*/
    0,                          /* tp_as_buffer */
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,                          /* tp_doc */
    (traverseproc)doctype_traverse, /*   tp_traverse */
    (inquiry)doctype_clear,        /*   tp_clear */
    0,                          /* tp_richcompare */
    0,                          /* tp_weaklistoffset */
};

