#ifndef __DOM__NODE__
#define __DOM__NODE__

#ifdef __cplusplus
extern "C" {
#endif

#include "Python.h"
#include "nodetype.h"

  /*Global Instances of the NodeTypes not defined in system*/
  extern PyObject *g_cdataSectionNodeType;
  extern PyObject *g_entityReferenceNodeType;
  extern PyObject *g_entityNodeType;
  extern PyObject *g_notationNodeType;

  /*Global Instances of the NodeNames*/
  extern PyObject *g_cdataSectionNodeName;


  /*Place holders for common thingies*/

  extern PyObject *g_emptyAttributes;
  extern PyObject *g_emptyChildNodes;
  extern PyObject *g_errorObject;

  extern long g_nodeCount;



#define PyDom_NODE \
    PyObject_HEAD \
    PyObject *parentNode; \
    PyObject *ownerDocument; \
    long docIndex;

  /* Just for access */
  typedef struct {
    PyDom_NODE
  } PyNodeObject;


  /*Node Methods */
  void Node_INIT(PyNodeObject *node, PyObject *ownerDocument, PyTypeObject *type);
  PyNodeObject *Node_AppendChild(PyNodeObject * self,PyNodeObject *child);
  PyNodeObject *Node_CloneNode(PyNodeObject *node,int deep,PyObject *newOwnerDocument);

  void node_dealloc(PyNodeObject *node);
  int node_clear(PyNodeObject *node);
  int node_traverse(PyNodeObject *self, visitproc visit, void *arg);
  PyObject *node_getattr(PyNodeObject *self, char *name,struct PyMethodDef methods[]);

  void *node_foreach_child(PyNodeObject *self,
                         int func(PyObject *node, void *data),
                         void *data);

  int node_test_ref_counts(PyObject *tester,PyNodeObject *node,long *childCtr,PyObject *internDict,int base);

  int node_to_xml(PyObject *node, PyObject *encoding);


  /*Functions exposed to python*/
  PyNodeObject *PyNode_AppendChild(PyNodeObject *self, PyNodeObject *args);
  PyObject *PyNode_normalize(PyObject * self, PyObject * args);
  PyObject *PyNode_hasChildNodes(PyObject * self, PyObject * args);
  PyObject *PyNode_removeChild(PyObject *self,PyObject *args);
  PyObject *PyNode_appendChild(PyObject *self,PyObject *args);
  PyObject *PyNode_insertBefore(PyObject *self,PyObject *args);
  PyObject *PyNode_replaceChild(PyObject *self,PyObject *args);
  PyObject *PyNode_cloneNode(PyObject *self,PyObject *args);
  PyObject *PyNode_isSameNode(PyObject *self, PyObject *args);



#define NODE_METHODS {"normalize",   (PyCFunction)PyNode_normalize,  METH_VARARGS, "Combine all neighboring text child nodes"}, \
                     {"hasChildNodes",   (PyCFunction)PyNode_hasChildNodes,  METH_VARARGS, "Return True if the node has children"}, \
                     {"removeChild",   (PyCFunction)PyNode_removeChild,  METH_VARARGS, "Remove a node from a child list"}, \
                     {"appendChild",   (PyCFunction)PyNode_appendChild,  METH_VARARGS, "Add a new child to the child list"}, \
                     {"insertBefore",   (PyCFunction)PyNode_insertBefore,  METH_VARARGS, "insert a new child to the child list"}, \
                     {"replaceChild",   (PyCFunction)PyNode_replaceChild,  METH_VARARGS, "replace a child in the child list"}, \
                     {"cloneNode",   (PyCFunction)PyNode_cloneNode,  METH_VARARGS, "Make a copy of a node from this document"}, \
                     {"isSameNode",   (PyCFunction)PyNode_isSameNode,  METH_VARARGS, "true if this node instance is the same as another"}






#ifdef __cplusplus
}
#endif


#endif
