/*
  Functions for stripping XML 
*/
#include "Python.h"
#include "debug.h"

#define ISWSPACE(x) ( ((x) == ' ') || ((x) == '\t' ) || ((x) == '\n') || ((x) == '\r') )

PyObject *_xmlStrip(PyUnicodeObject *self, int left, int right)
{
    Py_UNICODE *p = self->str;
    int start = 0;
    int end = self->length;

    if (left)
      while (start < end && ISWSPACE(p[start]))
            start++;

    if (right)
        while (end > start && ISWSPACE(p[end-1]))
            end--;

    if (start == 0 && end == self->length) {
        /* couldn't strip anything off, return original string */
        Py_INCREF(self);
        return (PyObject*) self;
    }
    return PySequence_GetSlice((PyObject *)self,start,end);
}


/* Why is such an approach (following dupe of perfectly working function above) needed ? */
/*static PyObject *brokenXmlStrip(PyUnicodeObject *self, int left, int right)
{
  wchar_t *buf;
  int start = 0, len = PyUnicode_GET_SIZE(self);
  int end = len;

  if ((buf = (wchar_t *)calloc(len, sizeof(wchar_t))) == NULL) {*/
    /* Exception: Out of memory */
  /*    return NULL;
  }
  len = PyUnicode_AsWideChar(self, buf, len);
  if (len == -1)
    return NULL;

  if (left)
    while (start < end && ISWSPACE(buf[start]))
      start++;

  if (right)
    while (end > start && ISWSPACE(buf[end-1]))
      end--;

  if (start == 0 && end == self->length) {
    couldn't strip anything off, return original string
    Py_INCREF(self);
    return (PyObject*) self;
  }
  return (PyObject*) PyUnicode_FromUnicode(self->str + start,
                                           end - start);
*/
/*   printf("%d %d\n",start,end); */
/*   if (left) */
/*     while (start < end && ISWSPACE(p[start])) { */
/*       printf("Chunking from left, %d %d %d %d\n",start,end,p[start],ISWSPACE(p[start])); */
/*       start++; */
/*     } */
/*   if (right) */
/*     while (end > start && ISWSPACE(p[end-1])) { */
/*       printf("Chunking from right, %d %d %d %d\n",start,end,p[end-1],ISWSPACE(p[end-1])); */
/*       end--; */
/*     } */
/*   if (start == 0 && end == self->length) { */
/*     couldn't strip anything off, return original string */
/*     Py_INCREF(self); */
/*     return (PyObject*) self; */
/*   } */
  
/*   return (PyObject*) PyUnicode_FromUnicode(p + start, */
/* 					   end - start */
/* 					   ); */
/*}*/


PyObject *PyXmlStrLStrip(PyObject * self, PyObject *args)
{
  PyObject *_argc0, *result;  /*Unicode objects*/

  if (!PyArg_ParseTuple(args,"O:XmlStrLStrip",&_argc0))
    return NULL;

  _argc0 = PyUnicode_FromObject(_argc0);
  if (!_argc0) {
    PyErr_SetString(PyExc_TypeError,
                    "XmlStrLStrip: Unable to convert argument to unicode");
    return NULL;
  }
  result = _xmlStrip((PyUnicodeObject *)_argc0, 1, 0);

  /*PyUnicode_FromObject always adds one REF so do a DECREF on it*/
  Py_DECREF(_argc0);

  /*Has a REF of one now so we don't need to INCREF it*/
  return result;
}


PyObject *PyXmlStrRStrip(PyObject * self, PyObject *args)
{
  PyObject *_argc0, *result;  /*Unicode objects*/

  if (!PyArg_ParseTuple(args,"O:XmlStrRStrip",&_argc0))
    return NULL;

  _argc0 = PyUnicode_FromObject(_argc0);
  if (!_argc0) {
    PyErr_SetString(PyExc_TypeError,
                    "XmlStrRStrip: Unable to convert argument to unicode");
    return NULL;
  }
  result = _xmlStrip((PyUnicodeObject *)_argc0, 0, 1);

  /*PyUnicode_FromObject always adds one REF so do a DECREF on it*/
  Py_DECREF(_argc0);

  /*Has a REF of one now so we don't need to INCREF it*/
  return result;
}



PyObject *PyXmlStrStrip(PyObject * self, PyObject *args)
{
  PyObject *_argc0, *result;  /*Unicode objects*/

  if (!PyArg_ParseTuple(args,"O:XmlStrStrip",&_argc0))
    return NULL;

  _argc0 = PyUnicode_FromObject(_argc0);
  if (!_argc0) {
    PyErr_SetString(PyExc_TypeError, 
                    "XmlStrStrip: Unable to convert argument to unicode");
    return NULL;
  }

  result = _xmlStrip((PyUnicodeObject *)_argc0, 1, 1);

  /*PyUnicode_FromObject always adds one REF so do a DECREF on it*/
  Py_DECREF(_argc0);

  /*Has a REF of one now so we don't need to INCREF it*/
  return result;
}


